# current weather
# summary forecast
# detailed forecast
# alerts

# text
# speak

import json
import pathlib
import threading
from dataclasses import dataclass
from datetime import datetime as dt
from typing import Tuple

import requests
from loguru import logger as LOGGER

from dt_tools.misc.helpers import ApiTokenHelper as api_helper
from dt_tools.misc.sound import Accent, Sound


AQI_DESC = {
    """
    Air Quality Index
    """
    -1: 'Unknown',
    1: 'Good',
    2: 'Moderate',
    3: 'Degraded',
    4: 'Unhealthy',
    5: 'Very Unhealthy',
    6: 'Hazardous',
}
WIND_DIRECTION_DICT = {
    "N": "North",
    "S": "South",
    "E": "East",
    "W": "West",
}

class WEATHER:
    API_KEY=cfg._WEATHER_API_KEY
    BASE_URL="http://api.weatherapi.com/v1" # 1 million calls per month
    CURRENT_URI="current.json"
    FORECAST_URI="forecast.json"
    SEARCH_URI="search.json"

@dataclass
class WeatherLocation():
    lat: float = 0.0
    long: float = 0.0
    location_name: str = None
    location_region: str = None

@dataclass
class WeatherConditions():
    location: WeatherLocation = None
    condition: str = None
    _condition_icon: str = None
    temp: float = None
    feels_like: float = None
    wind_direction: str = None
    wind_speed_mph: float = None
    wind_gust_mph: float = None
    humidity_pct: int = None
    cloud_cover_pct: int = None
    visibility_mi: float = None
    precipitation: float = None
    last_update: dt = None
    aqi: int = None
    aqi_text: str = None
    # _connect_retries: int = 0
    # _speak_thread_id: int = None
    # _speak_accent: Accent = Accent.UnitedStates
    # _disabled: bool = True

    def __post_init__(self):
        if self.lat == 0 or self.lon == 0:
            # Location not set, use (external) ip to set location
            external_ip = self._get_external_ip()
            if external_ip:
                self.lat, self.lon = self._get_lat_lon_from_ip(external_ip)
                LOGGER.warning(f'Location  not set, get via external IP [{external_ip}] - {self.lat_long}')
        self.refresh_if_stale()

    @property
    def accent(self) -> Accent:
        return self._speak_accent
    
    @accent.setter
    def accent(self, id: str):
        try:
            speak_accent = Accent[id]
        except Exception:
            LOGGER.warning(f'Conditions() invalid accent id [{id}], defaulting to US.')
            speak_accent = Accent.UnitedStates

        LOGGER.warning(f'Conditions() setting accent to: {speak_accent}')
        self._speak_accent = speak_accent

    @property
    def disabled(self) -> bool:
        return self._disabled
    
    @property
    def condition_icon(self) -> str:
        return self._condition_icon
    
    @condition_icon.setter
    def condition_icon(self, value):
        LOGGER.trace(f'icon: {value}')
        icon_filenm = value.replace('//cdn.weatherapi.com','./files/icons')
        icon_file = pathlib.Path(icon_filenm).absolute()
        if icon_file.exists():
            self._condition_icon = str(icon_file)
        else:
            LOGGER.error(f'Weather icon file does NOT exist - {icon_file}')
            self._condition_icon = None

    @property
    def lat_long(self) -> str:
        return f'{self.lat},{self.lon}'

    def to_string(self) -> str:
        degree = chr(176)
        text: str = f'Current weather conditions for {self.loc_name} {self.loc_region}. [{self.lat_long}]\n\n'
        text += f'{self.condition}\n'    
        text += f'  Temperature {self.temp}{degree} feels like {self.feels_like}{degree}\n'    
        text += f'  Wind {self.wind_speed_mph} mph - {self.wind_direction}\n'    
        text += f'  Humidity {self.humidity_pct}%\n'    
        text += f'  Cloud Cover {self.cloud_cover_pct}%, visability {self.visibility_mi} miles\n'    
        text += f'  Precipitation {self.precipitation} in.\n'    
        text += f'  Air Quality {self.aqi_text} [{self.aqi}]\n'    
        return text
    
    def refresh_if_stale(self, elapsed_mins: int = 15) -> bool:
        """
        Refresh weather data if stale.  Default is 15 monutes.
        """
        elapsed = "UNKNOWN"
        if self.last_update is not None:
            elapsed = (dt.now() - self.last_update).total_seconds() / 60
            if elapsed < elapsed_mins:
                LOGGER.trace('Weather data NOT refreshed')
                return False
        try:
            LOGGER.warning(f'- Weather being refreshed, last update {elapsed:.2f} minutes ago at {self.last_update}')            
        except Exception as ex:
            LOGGER.trace(f'no prior weather {ex}')
            LOGGER.warning('- Weather being refreshed, last update Unknown')

        target_url=f'{WEATHER.BASE_URL}/{WEATHER.CURRENT_URI}?key={WEATHER.API_KEY}&q={self.lat_long}&aqi=yes'
        LOGGER.debug(f'WEATHER url: {target_url}')
        try:
            resp = requests.get(target_url)
            if resp.status_code == 200:
                LOGGER.debug(json.dumps(resp.json(), indent=2))
                self._load_weather(resp.json())
                self._disabled = False
                return True
        except Exception as ex:
            LOGGER.warning('Unable to call weather api')
            LOGGER.warning(f'  URL   : {target_url}')
            LOGGER.warning(f'  ERROR : {repr(ex)}')
            self._connect_retries += 1
            if self._connect_retries > 3:
                LOGGER.error('Unable to reconnect to weather, disabled feature.')
                self._disabled = True
                # cfg.weather_enabled = False
            return False
                
        LOGGER.error(f'Request URL: {target_url}')
        LOGGER.error(f'Response status_code: {resp.status_code}')
        self._disabled = True
        # cfg.weather_enabled = False
        return False

    def speak_current_conditions(self) -> int:
        if self._speak_thread_id is not None and self._speak_thread_id > 0:
            LOGGER.warning('Speak thread in process... Ignoring request.')
            return False
        
        t = threading.Thread(target=self._speak_current_conditions_thread)
        t.start()
        self._speak_thread_id = t.native_id

    def _speak_current_conditions_thread(self):
        wind_direction = self._speak_direction(self.wind_direction)
        # cloud_cover_pct = self._speak_normalize_number(weather.cloud_cover_pct)
        temp = self._speak_normalize_number(self.temp)
        feels_like = self._speak_normalize_number(self.feels_like)
        humidity_pct = self._speak_normalize_number(self.humidity_pct)
        # precipitation = self._speak_normalize_number(weather.precipitation)
        visibility_mi = self._speak_normalize_number(self.visibility_mi)
        wind_speed_mph = self._speak_normalize_number(self.wind_speed_mph)
        wind_gust_mph = self._speak_normalize_number(self.wind_gust_mph)        
        time_now = dt.now().strftime("%I:%M%p")
        text = f'Current weather conditions at {time_now}.  '
        text += f'{self.condition}.  Temperature {temp}, feels like {feels_like}.  '
        text += f'{humidity_pct}% humidity, air quality is {self.aqi_text}.  '
        text += f'Visibility {visibility_mi} miles.  '
        text += f'Wind {wind_direction} {wind_speed_mph} mph, gusts up to {wind_gust_mph} mph.'
        ret = Sound().speak(text, speed=cfg.audio_speed, accent=self.accent)
        LOGGER.success('Speak current weather conditions complete.')
        self._speak_thread_id = None
        return ret

    def _speak_normalize_number(self, token) -> str:
        try:
            num = float(token)
            frac = num % 1
            resp = str(token).split('.')[0] if frac == 0 else token
        except Exception as ex:
            print(ex)
            resp = token 
        
        return resp

    def _speak_direction(self, token: str) -> str:
        resp = ''
        for char in token:
            resp += f' {WIND_DIRECTION_DICT[char]}'
        return resp.lstrip()

    def _load_weather(self, blob: dict):
        l_block: dict = blob['location']
        w_block: dict = blob['current']
        c_block: dict = w_block.get('condition',{})
        self.loc_name           = l_block.get('name', '')
        self.loc_region         = l_block.get('region', '')
        self.condition          = c_block.get('text','')  # w_block["condition"]["text"]
        self.condition_icon     = c_block.get('icon', '') # w_block["condition"]["icon"]
        self.temp               = float(w_block.get("temp_f", -1))
        self.feels_like         = float(w_block.get("feelslike_f", -1)) 
        self.wind_direction     = w_block.get("wind_dir", '')
        self.wind_speed_mph     = float(w_block.get("wind_mph", -1))
        self.wind_gust_mph      = float(w_block.get("gust_mph", -1))
        self.humidity_pct       = float(w_block.get("humidity", -1))
        self.cloud_cover_pct    = float(w_block.get("cloud", -1))
        self.visibility_mi      = float(w_block.get("vis_miles", -1))
        self.precipitation      = float(w_block.get("precip_in", -1))
        try:
            self.aqi                = int(w_block["air_quality"]['us-epa-index'])
        except Exception as ex:
            LOGGER.error(f'Unable to determine AQI: {repr(ex)}')
            self.aqi = -1
        self.aqi_text           = AQI_DESC[self.aqi]     
        self.last_update = dt.now()
        
    def _get_external_ip(self) -> str:
        # resp = requests.get('http://ifcfg.me')
        resp = requests.get('https://api.ipify.org')
        external_ip = resp.text
        LOGGER.debug(f'External IP identified as: {external_ip}')
        return external_ip
    
    def _get_lat_lon_from_ip(self, ip: str) -> Tuple[float, float]:
        lat: float = 0.0
        lon: float = 0.0
        url = f'https://ipapi.co/{ip}/latlong/'
        #headers = {'user-agent': 'ipapi.co/#ipapi-python-v1.0.4'} 
        headers = {'user-agent': 'ipapi.co/#custom'}               
        resp = requests.get(url, headers=headers)
        if resp.text.count(',') == 1:
            token = resp.text.split(',')
            lat = token[0]
            lon = token[1]
        LOGGER.warning(f'Lat/Lon identified - ip: {ip}  lat: {lat}  lon: {lon}')
        return lat, lon

def get_weather(lat: float, lon: float) -> WeatherConditions:
    loc = WeatherLocation(lat, lon)

def get_weather(street: str, city: str, state: str, zip: str) -> WeatherConditions:
    loc = xxxxx
    
if __name__ == "__main__":
    import dt_tools.logger.logging_helper as lh
    # lh.configure_logger(log_level='DEBUG', log_format=lh.DEFAULT_FILE_LOGFMT)
    # weather = Conditions()
    # LOGGER.info(f'\n{weather.to_string()}')
    # if ch().get_input_with_timeout("Speak current conditions (y/n) > ", ["y","n"],"n") == 'y':
    #     weather.speak_current_conditions()



import json
import pathlib
from dataclasses import dataclass, field
from pprint import pformat
from typing import Tuple, Dict

import requests
from loguru import logger as LOGGER
from dt_tools.misc.helpers import ApiTokenHelper as api_helper

class _GeoLoc_Control:
    # API_KEY = cfg._GEOLOC_API_KEY
    API_KEY = api_helper.get_api_token('geocode.maps.co')
    API_ENABLED = API_KEY is not None
    BASE_URL = 'https://geocode.maps.co'
    ADDRESS_URI = 'search'
    LAT_LON_URI = 'reverse'
    CACHE_FILENM = './files/geoloc_cache.json'

@dataclass
class LocationCache:
    cache: Dict[str, dict] = field(default_factory=dict)
    valid_cache: bool = True

    def __post_init__(self):
        # if not cfg.geoloc_enabled:
        if not _GeoLoc_Control.API_ENABLED:
            LOGGER.warning('GeoLoc API key not defined, cache lookup only.')

        self.load()
        if self.valid_cache:
            LOGGER.success(f'Location cache loaded with {len(self.cache)} entries.')
        else:
            LOGGER.warning('Cache does not exist and could not be created.')

    def __del__(self):
        if self.valid_cache:
            self.save()
            LOGGER.success(f'Location cache saved with {len(self.cache)} entries to {_GeoLoc_Control.CACHE_FILENM}')

    def save(self):
        if self.valid_cache:
            cache_file = pathlib.Path(_GeoLoc_Control.CACHE_FILENM)
            cache_file.write_text(json.dumps(self.cache,indent=2), encoding='UTF-8')

    def load(self):
        cache_file = pathlib.Path(_GeoLoc_Control.CACHE_FILENM)
        if cache_file.exists():
            self.cache = json.loads(cache_file.read_text(encoding='UTF-8'))
        elif cache_file.parent.exists():
            self.cache = {}
        else:
            self.valid_cache = False

    def exists(self, key) -> bool:
        return key in self.cache.keys()
    
    def get(self, key) -> dict:
        return self.cache.get(key, None)
    
    def add(self, key, data):
        if self.exists(key):
            LOGGER.error(f'{key} - ALREADY EXISTS IN CACHE')
        self.cache[key] = data

LOCATION_CACHE: LocationCache = LocationCache()

class GeoLocation:
    lat: float = None
    lon: float = None
    display_name: str = None
    house: int = None
    street: str = None
    city: str = None
    county: str = None
    state: str = None
    zip: int = None
    ip: str = None
    _json_payload: dict = None

    @property
    def address(self) -> str:
        if self.city is None and self.state is None:
            return ''
        address = ''
        if self.house:
            address += f'{self.house} '
        if self.street:
            address += f'{self.street}, '
        
        if self.city:
            address += f"{self.city}, "
        elif self.county:
            address += f'{self.county.replace("County","")}, '
        
        if self.state:
            address += f'{self.state}, '
        if self.country:
            address += f'{self.country.upper()}'

        return address
        
    @property
    def lat_lon(self) -> str:
        """Return latitude longitude as a comma seperated string"""
        if self.lat is None or self.lon is None:
            return ''
        return f'{float(self.lat):.5f},{float(self.lon):.5f}'
    
    def _clear_location_data(self):
        self.lat = None
        self.lon = None
        self.display_name = None
        self.house = None
        self.street = None
        self.city = None
        self.county = None
        self.state = None
        self.zip = None
        self.country = None
        self.ip = None
        self._json_payload = None
        LOGGER.debug('Location data cleared.')

    def get_location_via_lat_lon(self, lat: float, lon: float) -> bool:
        """Retrieve address location based on lat/lon coordinates"""
        self._clear_location_data()
        self.lat = lat
        self.lon = lon
        loc_dict = None
        if LOCATION_CACHE.exists(self.lat_lon):
            loc_dict = LOCATION_CACHE.get(self.lat_lon)
            LOGGER.info(f'({self.lat_lon}) retrieved from cache')
        elif not _GeoLoc_Control.API_ENABLED:
            return False
        else:
            url = f"{_GeoLoc_Control.BASE_URL}/{_GeoLoc_Control.LAT_LON_URI}?api_key={_GeoLoc_Control.API_KEY}&lat={self.lat}&lon={self.lon}"
            LOGGER.debug(f'GEOLOC url: {url}')
            try:
                resp = requests.get(url)
                if resp.status_code == 200:
                    LOGGER.trace(resp.json())
                    loc_dict = resp.json()
                    LOCATION_CACHE.add(self.lat_lon, loc_dict)
                    LOGGER.success(f'({self.lat_lon}) added to cache, {len(LOCATION_CACHE.cache)} total entries.')
            except Exception as ex:
                LOGGER.exception(f'Unable to get geoloc: {repr(ex)}')
                return False

        if loc_dict:
            self._json_payload = loc_dict
            addr_dict: dict = loc_dict['address']
            self.display_name = loc_dict['display_name']
            self.house = addr_dict.get('house_number', None)
            token = addr_dict.get('road', None)
            if token is None:
                token = addr_dict.get('street', None)
            self.street = token
            token = addr_dict.get('city', None)
            if token is None:
                token = addr_dict.get('own', None)
            if token is None:
                token = addr_dict.get('hamlet', None)
            self.city = token
            self.county = addr_dict.get('county', None)
            self.state = addr_dict.get('state', None)
            self.country = addr_dict.get('country_code', None)
            self.zip = addr_dict.get('postcode', None)
            # amenity: xxxx,  residential: cimarrone, village: Spring Lake, municipality: Spring Lake Township, neighbourhood: south slope, suburb: brooklyn, city: new york, state: new york
            return True
        
        LOGGER.error(f'Request URL: {url}')        
        LOGGER.error(f'Response status code: {resp.status_code}')
        return False
    
    def get_location_via_address_string(self, address: str) -> bool:
        """
        Retrieve location based on street address
        Required:
            address : typically house street, city, state, zip
        """
        if not _GeoLoc_Control.API_ENABLED:
            return False

        self._clear_location_data()
        url = f"{_GeoLoc_Control.BASE_URL}/{_GeoLoc_Control.ADDRESS_URI}?api_key={_GeoLoc_Control.API_KEY}&q={address}"
        LOGGER.debug(f'GEOLOC url: {url}')
        found = False
        resp = requests.get(url)
        if resp.status_code == 200:
            found = True
            json_data = resp.json()
            LOGGER.trace(json_data)
            if isinstance(json_data, dict):         
                loc_dict = json_data
            elif isinstance(json_data, list):
                if len(json_data) > 0:
                    loc_dict = resp.json()[0]
                else:
                    LOGGER.debug(f'GEOLOC not found for {address}')
                    found = False
            else:
                # LOGGER.warning(resp.json())
                found = False
        else:
            LOGGER.error(f'Request URL: {url}')        
            LOGGER.error(f'Response status code: {resp.status_code}')

        if found:
            self._json_payload = loc_dict
            self.lat = loc_dict['lat']
            self.lon = loc_dict['lon']
            self.display_name = loc_dict['display_name']

        return found
        

    def get_location_via_address(self, city: str, state: str, house: int=None, street: str=None, zip: int=None) -> bool:
        """
        Retrieve location based on street address
        Required:
            city, state
        Optional:
            house, street, zip
        """
        self._clear_location_data()
        if not _GeoLoc_Control.API_ENABLED:
            return False
        self.house = house
        self.street = street
        self.city = city
        self.state = state
        self.zip = zip
        return self.get_location_via_address_string(self.address)
    
    
    def get_location_via_zip(self, zip: str) -> bool:
        """Retrieve location based on zip code"""
        self._clear_location_data()
        self.zip = zip
        if not _GeoLoc_Control.API_ENABLED:
            return False

        url = f"{_GeoLoc_Control.BASE_URL}/{_GeoLoc_Control.ADDRESS_URI}?api_key={_GeoLoc_Control.API_KEY}&postalcode={self.zip}"
        LOGGER.debug(f'GEOLOC url: {url}')
        resp = requests.get(url)
        if resp.status_code == 200:
            LOGGER.trace(pformat(resp.json()))            
            loc_dict: dict = resp.json()[0]
            self.lat = loc_dict['lat']
            self.lon = loc_dict['lon']
            self.display_name = loc_dict['display_name']
            self._json_payload = loc_dict
            return True
        
        LOGGER.error(f'Request URL: {url}')        
        LOGGER.error(f'Response status code: {resp.status_code}')
        return False
    
    def get_location_via_ip(self) -> bool:
        """Retrieve location info based on IP address"""
        self._clear_location_data()
        # Retrieve public IP address
        resp = requests.get('http://ip-api.com/json/')
        resp_json = resp.json()
        if resp_json.get('status') == "success":
            ip = resp_json.get('query')
            self.lat = resp_json.get('lat')
            self.lon = resp_json.get('lon')
            self.city = resp_json.get('city')
            self.country = resp_json.get('countryCode')
            self.state = resp_json.get('region')
            self.zip = resp_json.get('zip')
            self.ip = ip
            self._json_payload = resp_json
            LOGGER.debug(f'External IP identified as: {ip}')
            return True
        
        self.lat = 0.0
        self.lon = 0.0
        LOGGER.error('Unable to determine ip for location identification')
        return False

    def GPS_dms_to_dd(lat_dms:Tuple[float, float, float], lat_ref: str, lon_dms: Tuple[float, float, float], lon_ref: str) -> Tuple[float, float]:
        """
        Return Lat/Lon decimal degrees (dd) from Lat/Long Degree, Minute Seconds (dms) coordinates
        Input:
            lat_dms: (degree, minute, sec)
            lat_ref: N or S
            lon_dmsL (degree, minute, sec)
            lon_ref: E or W
        Returns:
            lat_dd: decimal degree - latitude 
            lon_dd: decimal degree - longitude
            (0.0, 0.0) on ERROR
        """
        try:
            #          Degree   + (min/60)        + (seconds/3600)
            lat_dd = float(lat_dms[0]) + (float(lat_dms[1])/60) + (float(lat_dms[2])/3600)
            lon_dd = float(lon_dms[0]) + (float(lon_dms[1])/60) + (float(lon_dms[2])/3600)
            if lat_ref.lower() == "s":
                lat_dd *= -1
            if lon_ref.lower() == "w":
                lon_dd *= -1
        except Exception as ex:
            LOGGER.error(f'Lat/Lon: {repr(ex)}')
            lat_dd = 0.0
            lon_dd = 0.0

        lat_dd = float(f'{lat_dd:.4f}')
        lon_dd = float(f'{lon_dd:.4f}')
        
        return (lat_dd, lon_dd)

    def to_string(self) -> str:
        output = ''
        for attr in dir(self):
            value = getattr(self, attr)
            if value is not None and not callable(value) and not attr.startswith('_'):
                output += f'{attr:15} : {getattr(self, attr)}\n'
        
        return output

def print_object(obj):
    for line in obj.to_string().splitlines():
        LOGGER.info(f'  {line}')
    payload = getattr(obj,'_json_payload')
    if payload is not None:
        LOGGER.info('  Payload-')
        for k, v in payload.items():
            LOGGER.info(f'    {k:15} {v}')

if __name__ == "__main__":
    import dt_tools.logger.logging_helper as lh
    lh.configure_logger(log_level='INFO', brightness=False)
    helper = GeoLocation()
    address = '1812 Edgewood, Berkley, Mi,'
    helper.get_location_via_address_string(address)
    LOGGER.info(f'Address: {address}')
    LOGGER.info('Returns:')
    print_object(helper)

    print('')
    helper.get_location_via_ip()
    LOGGER.info(f'Location for {helper.ip}')
    LOGGER.info('Returns:')
    print_object(helper)


"""
This module demonstrates the Weather module functionality.


"""

from loguru import logger as LOGGER

import dt_tools.logger.logging_helper as lh
from dt_tools.os.os_helper import OSHelper

    
def demo():
    OSHelper.enable_ctrl_c_handler()
    LOGGER.info('-'*80)
    LOGGER.info('dt_misc_weather demo')
    LOGGER.info('-'*80)

    LOGGER.warning('TBD - Demo not ready...')

    LOGGER.info('')
    LOGGER.info('Demo complete.')

if __name__ == "__main__":
    lh.configure_logger(log_format=lh.DEFAULT_CONSOLE_LOGFMT, brightness=False)
    demo()



"""
_summary_

Raises:
    GeoLocationException: _description_

Returns:
    _type_: _description_
"""
from dataclasses import dataclass
from typing import List

import requests
from loguru import logger as LOGGER
import dt_tools.logger.logging_helper as lh

class GeoLocationException(Exception):
    def __init__(self, message):
        super(GeoLocationException, self).__init__(message)

@dataclass
class GeoLocationAddress():
    address: str
    latitude: float
    longitude: float
    _json_payload: dict

    def to_string(self) -> str:
        output = ''
        for attr in dir(self):
            value = getattr(self, attr)
            if value is not None and not callable(value) and not attr.startswith('_'):
                output += f'{attr:15} : {getattr(self, attr)}\n'
        
        return output

class GeoLocation():
    """
    GeoLocation helper class to help identify lat, long for addresses
    uses geocoding.geo.census.gov restAPI
    """
    
    _TARGET_URL="https://geocoding.geo.census.gov/geocoder/locations/address?street={street}&city={city}{parms}&benchmark=Public_AR_Current&format=json"
    
    def __init__(self):
        pass

    @classmethod
    def _get_url(cls, street, city, state, zipcode) -> str:
        url = cls._TARGET_URL.replace("{street}", street)
        parms = ''
        if city is not None:
            parms += f'&city={city}&state={state}'
        if zipcode is not None:
            parms += f'&zip={zipcode}'
        url = url.replace("{parms}", parms)
        LOGGER.debug(f'URL: {url}')
        return url

    # {"result": {
    #     "input": {
    #         "address": {"address": "4600 Silver Hill Rd, Washington, DC 20233"},
    #         "benchmark": {
    #             "isDefault": true,
    #             "benchmarkDescription": "Public Address Ranges - Current Benchmark",
    #             "id": "4",
    #             "benchmarkName": "Public_AR_Current"
    #         }
    #     },
    #     "addressMatches": [{
    #         "tigerLine": {
    #             "side": "L",
    #             "tigerLineId": "76355984"
    #         },
    #         "coordinates": {
    #             "x": -76.92748724230096,
    #             "y": 38.84601622386617
    #         },
    #         "addressComponents": {
    #             "zip": "20233",
    #             "streetName": "SILVER HILL",
    #             "preType": "",
    #             "city": "WASHINGTON",
    #             "preDirection": "",
    #             "suffixDirection": "",
    #             "fromAddress": "4600",
    #             "state": "DC",
    #             "suffixType": "RD",
    #             "toAddress": "4700",
    #             "suffixQualifier": "",
    #             "preQualifier": ""
    #         },
    #         "matchedAddress": "4600 SILVER HILL RD, WASHINGTON, DC, 20233"
    #     }]
    # }}

    @classmethod
    def lookup_address(cls, street:str, city:str = None, state:str = None, zipcd:str = None) -> List[GeoLocationAddress]:
        valid_parameters = True if street is not None and (zipcd is not None or (city is not None and state is not None)) else False
        if not valid_parameters:
            raise ValueError('Must supply street and zip or street, city and state')
        url = cls._get_url(street, city, state, zipcd)
        resp = requests.get(url)
        if resp.status_code != 200:
            msg = f"url: {url}\nInvalid Geolocation response code: {resp.status_code}"
            LOGGER.error(msg)
            raise GeoLocationException(msg)

        LOGGER.debug("Result:\n{}\n".format(resp.json()))
        location_list = []
        addr_matches = resp.json()['result']['addressMatches']
        for fnd_address in addr_matches:
            addr = fnd_address['matchedAddress']
            latitude = float(fnd_address['coordinates']['y'])
            longitude = float(fnd_address['coordinates']['x'])
            location = GeoLocationAddress(address=addr, latitude=latitude, longitude=longitude, _json_payload=fnd_address)
            location_list.append(location)

        LOGGER.debug(f'{len(location_list)} addresses identified.')
        return location_list

if __name__ == "__main__":
    lh.configure_logger(log_level="INFO")
    geo = GeoLocation()
    address = input("Street Address: ")
    while len(address) > 0:
        city = input("City   : ")
        state = input("State  : ")
        zipcd = input("Zip    : ")
        locations = geo.lookup_address(address, city, state, zipcd)
        for loc in locations:
            LOGGER.info(f'{loc.latitude:.4f}/{loc.longitude:.4f} - {loc.address}')
        address = input("Street Address (blank to quit): ")